--[[
Patch Export - v1.0.2
Created by Jason Giaffo
Contact: GiaffoDesigns.com/contact
Last updated May 1, 2017
--]]



---- CONFIG ----
local range  = {}

range.start  = 0		--starting universe number; enter 0 to have plugin ask
range.finish = 0		--final exported universe; enter 0 to have plugin ask



---- FUNCTIONS ----
local unpack = table.unpack



function math.trunc(num, mod)
  local x = num - (num%mod)
  return x
end



local function msgbox(title, message_box, message_text)
  -- v. 1.0
  -- function avoids using confirmation box function with version 3.1.2.5, where it doesn't exist
  local confirm_method
  
  local version = gma.show.getvar('version')
  
  if version:find('3.1.2') == 1 then 
    confirm_method = 'textinput'
  else 
    confirm_method = 'box' end
  
  if confirm_method == 'box' then
    gma.gui.msgbox(title, message_box)
  elseif confirm_method == 'textinput' then
    gma.textinput(title, message_text)
  end
end



local function CSVlineToArray(str)
  local t = {}
  local indices = {1, 1}
  local remaining = true
  local restart = 1
  while remaining == true do
    local pt = {str:find(',', restart)}		--find next comma at or after restart point
    if pt[1] then
	  t[#t+1] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
	  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
    else
	  remaining = false
	  t[#t+1] = str:sub(restart, str[-1])
	end
  end

  return t
end



function writeCSV(...)
  local list = {...}

  local t = list[1]
  if #list > 1 then
    for i = 2, #list do
	  t = t..','..list[i]
	end
  end
  
  if t == nil then t = '' end
  
  return t
end



function writeCSVtable(list)
  local t = ''
  for i = 1, #list do
    t = t..writeCSV(unpack(list[i]))..'\n'
  end

  return t
end



local function address_to_array(str)
  local index = str:find(':')
  local t = {}
  t[1] = str:sub(1, index-1)
  t[2] = str:sub(index+1, #str)

  return t
end


local function formatAddress(addr)
	local t = addr:sub(unpack({addr:find('%S+')}))		--extract text without spaces
	t = t:gsub(':', '.')								--swap colon for decimal
	
	return t
end

















return function()


---- FILESYSTEM ----
local fn = {}
fn.tempreport  = 'tempreport.csv'
fn.finalreport = 'dmx_report '..tostring(math.trunc(gma.gettime(), 1))..'.csv'

local dir = {}
dir.main    = gma.show.getvar('PATH')..'/'
dir.reports = dir.main..'reports/'

local file = {}
file.tempreport  = dir.reports..fn.tempreport
file.finalreport = dir.reports..fn.finalreport





if range.start == 0 then
	range.start  = tonumber(gma.textinput('Starting Universe', '[ENTER STARTING UNIV #]'))
	if not range.start then goto EOF end
end

if range.finish == 0 then
	range.finish = tonumber(gma.textinput('Ending Universe', '[ENTER FINAL UNIV #]'))
	if not range.finish then goto EOF end
end


range.total = range.finish - range.start + 1
range.str    = 'Dmxuniverse '..range.start..'.*'
if range.finish > range.start then
  for i = range.start+1, range.finish do
	range.str = range.str..' + '..i..'.*'
  end
end


-- export from MA
gma.cmd('SelectDrive 1')
gma.cmd('List '..range.str..' /f=\"'..fn.tempreport..'\"')

local reportBasic = {}
--columns are: [1]DMX Label, [2]DMX Address (x:xxx), [3]Fixture Name, [4]FixID, [5]ChID, [6]Attrib, [7]Type, [8]DMX Profile

--import into script as array
local pBar = gma.gui.progress.start('importing DMX patch')
gma.gui.progress.setrange(pBar, 0, (range.total*512 + 1))
local ct = 1

for line in io.lines(file.tempreport) do
  reportBasic[#reportBasic+1] = CSVlineToArray(line)
  gma.gui.progress.set(pBar, ct)
  ct = ct + 1
end

gma.gui.progress.stop(pBar)


  

local reportFinal = {}
reportFinal[1] = {'addrStart', 'Fixture Name', 'Footprint', 'FixID', 'ChID'}
local line = 2 --set starting line at 2 to avoid reading the column headers as patch
local remaining = true

local pBar = gma.gui.progress.start('Converting to CSV')
gma.gui.progress.setrange(pBar, 0, (range.total*512 + 1))

while remaining == true do
  local addrStart = reportBasic[line][2]:sub(unpack({reportBasic[line][2]:find("%S+")}))			--inserts address start into array with spaces removed
  local addrFinish
  local addrRange
  
  local nameCurrent = reportBasic[line][3]
  local fixID, chID = reportBasic[line][4], reportBasic[line][5]
  
  
	for i = line, #reportBasic do
	  if i == #reportBasic then remaining = false end
	  gma.gui.progress.set(pBar, line)
	  
	  if reportBasic[line][3] == nameCurrent and tonumber(address_to_array(reportBasic[line][2])[2]) ~= 512 then				--
	    line = line + 1
	  elseif reportBasic[line][3] ~= nameCurrent then
		addrFinish = reportBasic[line-1][2] --where name has changed, finishing line is the line before the name change
		break
	  elseif tonumber(address_to_array(reportBasic[line][2])[2]) == 512 then --where DMX address 512 has been reached, finishing line is the current line
		addrFinish = reportBasic[line][2]
		line = line + 1
		break
	  else
	    gma.echo('loop not triggered')
	  end
	  
	  
	end
  local addrRange = tonumber(address_to_array(addrFinish)[2]) - tonumber(address_to_array(addrStart)[2]) + 1
  
  reportFinal[#reportFinal+1] = {formatAddress(addrStart), nameCurrent, addrRange, fixID, chID}
  if tonumber(address_to_array(addrFinish)[2]) == 512 then reportFinal[#reportFinal+1] = {} end
end

gma.gui.progress.stop(pBar)


local writeFile = io.open(file.finalreport, 'w')
writeFile:write(writeCSVtable(reportFinal))
writeFile:close()

os.remove(file.tempreport)

if string.lower(gma.show.getvar('hostsubtype')) == 'onpc' then			--open file in OnPC
  os.execute('\"'..file.finalreport..'\"') end


::EOF::
end --EOF